// ---
// 24 Jun 97
//
// Since operator is a keyword in C++,
// we should not use it as a variable
// name in C.
//

// ---
// Saturday   14:32:12 PDT   14 June 1986
// Tuesday    08:41:53 PST   24 February 1987
// Sunday     20:20:11 PST   27 December 1987
//
// All end-of-quarter grading activities are made available here.
//
// The information used to assign final grades is
// obtained from the file named <database>_assign_grades,
// where <database> is the name of the database.
//
// Typical lines in the file <database>_assign_grades are:
//
//    Needed to pass the course:
//   
//      final >= 55  &&  class total >= 630
//   
//    Grades are then based on the class total:
//   
//       A*** =   highest class grade
//       A++  >=  886
//       A    >=  802
//       B+   >=  754
//       B    >=  697
//       C    >=  630
//

#include "grade.h"

#define   QUIT   4                            /* highest option number */

struct data {
   char*    oper;                                   /* either && or || */
   char     highest_grade_name[maxword];             /* example:  A*** */
   char**   grade_name;
   dbl*     grade_boundry;
   dbl      minimal_final_score;
   dbl      minimal_class_total;
   int      max_grade_name_length;
   int      n;                 /* size of grade_name and grade_boundry */
};

typedef   struct data               data;
typedef   enum {failure, success}   outcome;

static int       get_local_option(int* test_mode_ptr);
static void      assign_final_grades(cchr* database, data* dp, int test_mode);
static outcome   get_data_to_assign_grades(data* dp, FILE* ifp, char* f_name);
static outcome   prn_data_to_assign_grades(data* dp, char* filename);
static void      compute_final_grades(cchr* database, data* dp);
static void      prn_general_assign_grades_info(void);

void assign_grades(cchr* database)
{
   int     option = 0;
   int     test_mode;
   data    d;
   data*   dp = &d;                      /* pass to wrt_evaluations()? */

   while (option != QUIT) {
      option = get_local_option(&test_mode);
      switch (option) {
      case 0:
         printf("PROGRAMMER ERROR: Option 0 already processed.\n\n");
         break;
      case 1:
         assign_final_grades(database, dp, test_mode);
         break;
      case 2:
         translate_letter_grades(database, test_mode); /* in f_decls.h */
         break;
      case 3:
         wrt_evaluations(database, test_mode);         /* in f_decls.h */
         break;
      case QUIT:
         break;
      default:
         printf("\nCANNOT GET HERE: In assign_grades() - bye.\n\n");
         exit(1);
      }
   }
}

static int get_local_option(int* test_mode_ptr)
{
   char   c;
   char   answer[maxline];
   int    option = 0;

   printf("%s\n%s\n%s\n%s\n%s\n%s\n\n%s",
      "Options:",
      "   0  information",
      "   1  assign final grades",
      "   2  translate letter grades",
      "   3  write evaluations",
      "   4  quit",
      "Enter your option: ");
   get_string_reply(answer);               /* answer has been stripped */
   putchar('\n');
   if (*answer == 'i')
      option = 0;
   else if (*answer == 'a')
      option = 1;
   else if (*answer == 't')
      option = 2;
   else if (*answer == 'w')
      option = 3;
   else if (*answer == 'q')
      option = QUIT;
   else
      option = extract_int(answer, 0, QUIT);
   if (option == 0) {
      prn_general_assign_grades_info();
      return (get_local_option(test_mode_ptr));
   }
   else if (option != QUIT) {
      printf("Do you wish to be in test mode?  ");
      c = reply();
      putchar('\n');
      *test_mode_ptr = (c == 'y') ? 1 : 0;
   }
   return option;
}

static void assign_final_grades(cchr* database, data* dp, int test_mode)
{
   char      filename[maxword];
   int       dummy = test_mode;                // keep the compiler happy
   FILE*     ifp;
   outcome   result;

   // ---
   // Keep the g++ compiler happy.
   //
   ++dummy;
   sprintf(filename, "%s_assign_grades", database);
   printf("Do you want instructions?  ");
   if (reply() == 'y') {
      putchar('\n');
      a_f_g_info(filename);
      printf("Are you ready to continue?  ");
      if (reply() == 'n') {
         putchar('\n');
         return;
      }
   }
   putchar('\n');
   if ((ifp = fopen(filename, "r")) == NULL) {
      printf("Cannot open %s.\n\n", filename);
      a_f_g_info(filename);
      return;
   }
   result = get_data_to_assign_grades(dp, ifp, filename);
   gfclose(ifp);
   if (result == failure)
      return;
   if (prn_data_to_assign_grades(dp, filename) == failure)
      return;
   compute_final_grades(database, dp);
}

static outcome get_data_to_assign_grades(data* dp, FILE* ifp, char* filename)
{
   char    grade_name[maxword];
   char    highest_grade_name[maxword];
   char    line[maxline];
   char*   p;
   int     i;
   int     length;
   int     max_length;
   int     n;
   long    offset;

   while ((p = fgets(line, maxline, ifp)) != NULL
      && strstr(line, "Needed to pass") == NULL)
         ;
   if (p == NULL) {
      printf("ERROR:  Cannot find \"Needed to pass ...\"\n\n");
      a_f_g_info(filename);
      return failure;
   }
   while ((p = fgets(line, maxline, ifp)) != NULL
      && (p = strstr(line, "final")) == NULL)
         ;
   if (p == NULL) {
      printf("ERROR:  Cannot find \"final >= ...\"\n\n");
      a_f_g_info(filename);
      return failure;
   }
   if ((p = strstr(p, ">=")) == NULL) {
      printf("ERROR:  Cannot find \"final >= ...\"\n\n");
      a_f_g_info(filename);
      return failure;
   }
   for (p += 2; isspace(*p); ++p)
      ;
   if (sscanf(p, "%lf", &(dp -> minimal_final_score)) != 1) {
      printf("ERROR:  Cannot find score after \"final >= ...\"\n\n");
      a_f_g_info(filename);
      return failure;
   }
   for ( ; !isspace(*p); ++p)                  /* move past the number */
      ;
   for ( ; isspace(*p); ++p)                   /* move past the blanks */
      ;
   if (strncmp(p, "&&", 2) == 0)
      dp -> oper = "&&";
   else if (strncmp(p, "||", 2) == 0)
      dp -> oper = "||";
   else {
      printf("ERROR:  Cannot find operator && or ||.\n\n");
      a_f_g_info(filename);
      return failure;
   }
   if ((p = strstr(p + 2, "class total")) == NULL) {
      printf("ERROR:  Cannot find \"class total >= ...\"\n\n");
      a_f_g_info(filename);
      return failure;
   }
   if ((p = strstr(p, ">=")) == NULL) {
      printf("ERROR:  Cannot find \"class total >= ...\"\n\n");
      a_f_g_info(filename);
      return failure;
   }
   for (p += 2; isspace(*p); ++p)
      ;
   if (sscanf(p, "%lf", &(dp -> minimal_class_total)) != 1) {
      printf("ERROR:  Cannot find score after \"class total >= ...\"\n\n");
      a_f_g_info(filename);
      return failure;
   }
   while ((p = fgets(line, maxline, ifp)) != NULL
      && strstr(line, "Grades are then based") == NULL)
         ;
   if (p == NULL) {
      printf("ERROR:  Cannot find \"Grades are then based ...\"\n\n");
      a_f_g_info(filename);
      return failure;
   }
   offset = ftell(ifp);
   while ((p = fgets(line, maxline, ifp)) != NULL
      && (p = strstr(line, "highest class grade")) == NULL)
         ;
   if (strncmp(p, "highest class grade", 19) == 0) {
      if (sscanf(line, "%s", highest_grade_name) == 1)
         strcpy(dp -> highest_grade_name, highest_grade_name);
      else {
         printf("CONVERSION ERROR: String conversion failed.\n\n");
         a_f_g_info(filename);
         return failure;
      }
   }
   else
      *(dp -> highest_grade_name) = '\0';
   fseek(ifp, offset, 0);
   n = 0;
   while (fgets(line, maxline, ifp) != NULL)
      if (strstr(line, ">=") != NULL)
         ++n;
   if (n <= 0) {
      printf("ERROR:  List of grade names and grade boundries missing.\n\n");
      a_f_g_info(filename);
      return failure;
   }
   dp -> grade_name = (char **) gcalloc(n, sizeof(char *));
   dp -> grade_boundry = (double *) gcalloc(n, sizeof(double));
   dp -> n = n;
   fseek(ifp, offset, 0);
   i = length = max_length = 0;
   while (fgets(line, maxline, ifp) != NULL) {
      if ((p = strstr(line, ">=")) != NULL) {
         sscanf(line, "%s", grade_name);
         length = strlen(grade_name);
         dp -> grade_name[i] = (char *) gcalloc(length + 1, sizeof(char));
         strcpy(dp -> grade_name[i], grade_name);
         if (length > max_length)
            max_length = length;
         if (sscanf(p + 2, "%lf", &(dp -> grade_boundry[i])) != 1) {
            printf("%s\n\n%s%s\n",
               "ERROR:  A grade boundry is missing.",
               "Offending line:  ", line);
            a_f_g_info(filename);
            return failure;
         }
         ++i;
      }
   }
   dp -> max_grade_name_length = max_length;
   return success;
}

static outcome prn_data_to_assign_grades(data* dp, char* filename)
{
   int    c;
   int    i;
   int    m;

   printf("%s%s%s\n\n%s\n\n%s%.1f%s%s%s%.1f\n\n%s\n\n",
      "Here is the data retrieved from the file ", filename, ":",
      "   Needed to pass the course:",
      "      final >= ", dp -> minimal_final_score,
      "     ", dp -> oper, "     class total >= ",
      dp -> minimal_class_total,
      "   Grades are then based on the class total:");
   if (*(dp -> highest_grade_name) != '\0') {
      m = strlen(dp -> highest_grade_name);
      if (m < dp -> max_grade_name_length)
         m = dp -> max_grade_name_length;
      printf("%s%-*s%s\n",
         "      ", m, dp -> highest_grade_name, "  =   highest class grade");
   }
   else
      m = dp -> max_grade_name_length;
   for (i = 0; i < dp -> n; ++i)
      printf("%s%-*s%s%.1f\n",
         "      ", m, dp -> grade_name[i], "  >=  ", dp -> grade_boundry[i]);
   printf("\nIs this correct?  ");
   c = reply();
   putchar('\n');
   if (c == 'y' || c == '\n')
      return success;
   else {
      printf("CORRECTIONS NEEDED: Please edit the file %s.\n\n", filename);
      return failure;
   }
}

static void compute_final_grades(cchr* database, data* dp)
{
   char*   cp;
   char    current_name_line[maxline];
   char    line[maxline];
   char    tfn[maxstring];
   int     i;
   int     j;
   int     k;
   int     n;
   int     nn;
   int     ct_index;
   int     f_index;
   int     grade_line_printed;
   dbl     class_total;
   dbl     final;
   dbl     highest_class_total;
   long    offset;
   FILE*   ifp;
   FILE*   ofp;
   LINE*   hp;
   LINE*   p;

   tmpnam(tfn);
   startup(database, &ifp, tfn, &ofp);
   hp = cll_nail(ifp);
   n = cnt_ll_lines(hp);
   f_index = ct_index = 0;
   for (i = 0, p = hp; p != NULL; ++i, p = p -> next)
      if (strstr(p -> line, "final:"))
         f_index = i;
      else if (strstr(p -> line, "class total:"))
         ct_index = i;
   rll_lines(hp);
   if (f_index == 0) {
      printf("ERROR:  The tag \"final:\" was not found.\n\n");
      wrapup(database, ifp, tfn, ofp);
      return;
   }
   if (ct_index == 0) {
      printf("ERROR:  The tag \"class total:\" was not found.\n\n");
      wrapup(database, ifp, tfn, ofp);
      return;
   }
   /***=
     *   Next, run through the header lines and set the offset.
   =***/
   offset = ftell(ifp);
   while (fgets(line, maxline, ifp) != NULL && !isname(line)) {/* headers */
      fputs(line, ofp);
      offset = ftell(ifp);
   }
   fseek(ifp, offset, 0);
   highest_class_total = 0.0;
   for (i = 0; fgets(line, maxline, ifp) != NULL; ++i) {
      if (i % n == ct_index) {
         cp = strstr(line, "class total:") + 12;
         if (sscanf(cp, "%lf", &class_total) == 1
            && class_total > highest_class_total)
               highest_class_total = class_total;
      }
   }
   nn = i;                     /* count of all the name and info lines */
   fseek(ifp, offset, 0);
   for (i = 0; i < nn; i += n) {
      fgets(current_name_line, maxline, ifp);/* save for error reporting */
      fputs(current_name_line, ofp);
      for (j = 1; j < n; ++j) {
         fgets(line, maxline, ifp);                       /* info line */
         fputs(line, ofp);
         if (j == f_index) {
            cp = strstr(line, "final:") + 6;
            if (sscanf(cp, "%lf", &final) != 1)
               final = 0.0;
         }
         else if (j == ct_index) {
            cp = strstr(line, "class total:") + 12;
            if (sscanf(cp, "%lf", &class_total) != 1)
               class_total = 0.0;
         }
      }
      /***=
        *   Check for F grade first.
      =***/
      grade_line_printed = 0;                                 /* false */
      if (strcmp(dp -> oper, "&&") == 0
         && (final < dp -> minimal_final_score
            || class_total < dp -> minimal_class_total)) {
               fprintf(ofp, "   grade:  F\n");
               grade_line_printed = 1;                         /* true */
         }
      else if (strcmp(dp -> oper, "||") == 0
         && (final < dp -> minimal_final_score
            && class_total < dp -> minimal_class_total)) {
               fprintf(ofp, "   grade:  F\n");
               grade_line_printed = 1;                         /* true */
         }
      else if (class_total == highest_class_total) {
            fprintf(ofp, "   grade:  %s\n", dp -> highest_grade_name);
            grade_line_printed = 1;                            /* true */
      }
      else {
         for (k = 0; k < dp -> n; ++k) {
            if (class_total >= dp -> grade_boundry[k]) {
               fprintf(ofp, "   grade:  %s\n", dp -> grade_name[k]);
               grade_line_printed = 1;                         /* true */
               break;
            }
         }
      }
      if (!grade_line_printed) {
         printf("%s\n\n%s%s\n",
            "ERROR: Grade line not printed.",
            "Current name line: ", current_name_line);
         exit(1);
      }
   }
   wrapup(database, ifp, tfn, ofp);
}

static void prn_general_assign_grades_info(void)
{
   printf("%s%s\n\n",
      "WARNING: The \"write evaluations\" option ",
      "has not been implemented yet.");
}
