#include "grade.h"

// ---
// Correct mismatched ending blank lines in database via match_ebl().
//    There could be, for example, too many blank lines at the end
//    of the database.
//
// Check for the following errors:
//    no name lines in the database,
//    inconsistent kinds of info lines,
//    inconsistent numbers of scores.
//
// REMEMBER:
//    A file is a FIFO object and the pointer ifp points
//    to the file itself, not a place in the file.
//
// cll_nail := create a linked list of name and info lines
//

int chk_for_errors(cchr* database)
{
   char*    lp;                                           // line pointer
   char**   t;                                                // template
   char**   w;                                              // work space
   int      error = 0;
   int      error1;
   int      error2;
   int      error3;
   int      error4;
   int      i;
   int      j;
   int      n;
   FILE*    ifp;
   LINE*    hp;

   ifp = gfopen(database, "r");
   if (ifp == NULL) {
      cout << "WHOOPS AGAIN: Cannot open " << database << "!\n\n";
      cout.flush();
      return 777;
   }
   ifp = match_ebl(database, ifp);
   hp = cll_nail(ifp);
   n = cnt_ll_lines(hp);
   if (n == 0) {
      printf("There are no name lines in your database - bye!\n\n");
      return (error = 1);
   }
   t = (char **) gcalloc((unsigned) n, sizeof(char *));
   w = (char **) gcalloc((unsigned) n, sizeof(char *));
   for (i = 0; i < n; ++i) {
      t[i] = (char *) gcalloc((unsigned) maxline, sizeof(char));
      w[i] = (char *) gcalloc((unsigned) maxline, sizeof(char));
   }
   /***=
     *   Space now allocated for template and work.
     *   Fill the template next.
   =***/
   while (fgets(t[0], maxline, ifp) != NULL && !isname(t[0]))
      ;      /* run through header lines - put first name line in t[0] */
   for (i = 1; i < n; ++i)
      fgets(t[i], maxline, ifp);
   /***=
     *   The template is filled.
     *
     *   Now we repeatedly fill the work space
     *   and check it against the template.
   =***/
   while ((lp = fgets(w[0], maxline, ifp)) != NULL) {
      for (i = 1; i < n && lp != NULL; ++i)
         lp = fgets(w[i], maxline, ifp);
      if (!isname(w[0]) || i != n) {
         printf("\n\n%s\n%s\n\n",
            "SEVERE ERROR: Mismatched numbers of info lines.",
            "Check your database in the vacinity of trouble lines.");
         printf("Template:\n\n");
         for (j = 0; j < n; ++j)
            printf("%s", t[j]);
         printf("\nTrouble:\n\n");
         for (j = 0; j < i; ++j)                          // only i lines
            printf("%s", w[j]);
         printf("\n");
         return (error = 1);
      }
      for (i = 1; i < n && !error; ++i) {
         error1 = isname(w[i]);
         error2 = isscore(w[i]) != isscore(t[i]);
         error3 = issimple_score(w[i]) != issimple_score(t[i]);
         error4 = istotal_score(w[i]) != istotal_score(t[i]);
         error = error1 || error2 || error3 || error4;
      }
      if (error) {
         printf("\n\n%s\n\n",
            "SEVERE ERROR: Inconsistent kinds of info lines.");
         printf("Template:\n\n");
         for (j = 0; j < n; ++j)
            printf("%s", t[j]);
         printf("\nTrouble:\n\n");
         for (j = 0; j < n; ++j)
            printf("%s", w[j]);
         printf("\n");
         return (error);
      }
      for (i = 1; i < n && !error; ++i)
         if (issimple_score(t[i]))
            error = score_cnt(t[i]) != score_cnt(w[i]);
      if (error) {
         printf("\n\n%s\n\n",
            "SEVERE ERROR: Inconsistent number of scores.");
         printf("Template:\n\n");
         for (j = 0; j < n; ++j)
            printf("%s", t[j]);
         printf("\nTrouble:\n\n");
         for (j = 0; j < n; ++j)
            printf("%s", w[j]);
         printf("\n");
         return (error);
      }
   }
   gfclose(ifp);
   for (i = 0; i < n; ++i) {
      free(t[i]);
      free(w[i]);
   }
   free(t);
   free(w);
   return 0;                                      // no errors were found
}

// ---
// Following each name line must be exactly one info line:
//
//    NAME: EDELSON, DANIEL ROSS   ID: 566556382  F86-COURSE: 40900  ...
//    a+ a+ c a+
//    NAME: GAVER, HARRY LAWRENCE  ID: 175369005  F86-COURSE: 40900  ...
//    a a b+ a
//
// Typically the database is the class schedule number.
// Example:  40900
//

int chk_for_wrt_eval_errors(cchr* database)
{
   char         filename[maxword];
   char         line[maxline];
   char*        q;
   int          cnt;
   int          nrepl;                       /* number of replacements */
   FILE*        ifp;
   INFO_LINE*   hp;
   INFO_LINE*   p;

   ifp = gfopen(database, "r");
   hp = cll_info_lines(ifp);
   for (cnt = 0, p = hp; p != NULL; ++cnt, p = p -> next)
      ;
   if (cnt != 1) {
      printf("%s%d%s\n%s\n\n",
         "The number of info lines is ", cnt, ".",
         "There must be only one info line.");
      exit(1);
   }
   nrepl = word_cnt(hp -> line);
   rll_lines(hp);
   while (fgets(line, maxline, ifp) != NULL && !isname(line))
      ;                                /* run through the header lines */
   while (fgets(line, maxline, ifp) != NULL) {
      if (word_cnt(line) != nrepl) {
         printf("%s\n\n%s\n%s%d%s\n%s%d%s\n\n",
            "ERROR on the following line:",
            line,
            "The word count is ", word_cnt(line), ".",
            "It should be ", nrepl, ".");
         exit(1);
      }
      fgets(line, maxline, ifp);              /* discard the name line */
   }
   gfclose(ifp);
   /***=
     *   Now check that the number of replacements
     *   is consistent with the corresponding template file.
     *   See wrt_evaluation.c for an example of a template file.
   =***/
   sprintf(filename, "%s_template", database);
   ifp = gfopen(filename, "r");
   while (fgets(line, maxline, ifp) != NULL && strstr(line, ":::::") == NULL)
      ;
   cnt = 0;
   while (fgets(line, maxline, ifp) != NULL) {
      for (q = line; *q != '\0'; ++q)
         if (*q == '$')                     /* each $ must be replaced */
            ++cnt;
   }
   if (cnt != nrepl) {
      printf("%s%s%s\n%s\n%s\n%s%s\n\n",
         "ERROR: In the file ", filename, " the number of words",
         "       on each info line does not equal the number",
         "       of replacements ((indicated by $)) in the",
         "       file ", filename);
      exit(1);
   }
   gfclose(ifp);
   return nrepl;
}
