#include "grade.h"

// ---
// For each score a count and average is computed.
// If the name of the database is class, then this
// information is passed to prn_statistics for printing
// in the file class_statistics.
//
// cll_score_lines := create a linked list of score lines
//
// nsl             := number of score lines
// go[i]           := number of scores on (i + 1)st score line
// cnt_array[i][j] := count of scores corresponding to the
//                       (i + 1)st score line, (j + 1)st position
// avg_array[i][j] := avg of scores corresponding to the
//                       (i + 1)st score line, (j + 1)st position
//

void compute_statistics(cchr* database)
{
   char          line[maxline];
   char*         lp;
   char*         s;
   int           cnt;
   int**         cnt_array;
   int*          go;
   int           i;
   int           j;
   int           n;
   int           nsl = 0;                     /* number of score lines */
   dbl           score;
   dbl**         avg_array;
   FILE*         ifp;
   SCORE_LINE*   head;
   SCORE_LINE*   p;

   printf("Computing statistics ...\n\n");
   ifp = gfopen(database, "r");
   head = cll_score_lines(ifp);
   for (p = head; p != NULL; p = p -> next)
      ++nsl;
   go = (int *) gcalloc((unsigned) nsl, sizeof(int));
   cnt_array = (int **) gcalloc((unsigned) nsl, sizeof(int *));
   avg_array = (dbl **) gcalloc((unsigned) nsl, sizeof(dbl *));
   for (i = 0, p = head; p != NULL; ++i, p = p -> next) {
      go[i] = n = score_cnt(p -> line);
      cnt_array[i] = (int *) gcalloc((unsigned) n, sizeof(int));
      avg_array[i] = (dbl *) gcalloc((unsigned) n, sizeof(dbl));
   }
   rewind(ifp);
   while ((lp = fgets(line, maxline, ifp)) != NULL && !isname(line))
      ;                                /* run through the header lines */
   while (lp != NULL && !isscore(line))
      lp = fgets(line, maxline, ifp);    /* run through nonscore lines */
   while (lp != NULL && isscore(line)) {
      for (i = 0; i < nsl; ++i) {
	 s = strchr(line, ':') + 1;
	 s = skip_space(s);
	 for (j = 0; j < go[i]; ++j) {
	    if (sscanf(s, "%lf", &score) == 1) {
	       cnt = cnt_array[i][j] += 1;
	       avg_array[i][j] += (score - avg_array[i][j]) / (dbl) cnt;
	    }
	    SET(s);
	 }
	 while ((lp = fgets(line, maxline, ifp)) && !isscore(line))
	    if (i < nsl - 1)
	       assert(lp != NULL);
      }
   }
   gfclose(ifp);
   prn_statistics(database, head, cnt_array, avg_array, go, nsl);
}

// ---
// The following kinds of totals can be computed:
//    ((cut k)) total
//    ((cut k)) weighted total
//    ((cut k)) weighted ordered total
//
// Mnemonics:
//
//    a[]  := array obtained from answer
//    go[] := array obtained from a[]
//            it marks which info line to use and how many scores to obtain
//
//    m  := number of selected score lines
//    m  := size of array a[] which contains selected score line numbers
//    n  := number of info lines
//    nn := n + 1, so that subscripts will match info line numbers
//    nn := size of g[] and of cut[]
//
//    true: 1 <= m && m <= n
//
//    cll_info_lines := create a linked list of info lines
//    prn_ll_lines   := print a linked list of lines
//

int compute_total_scores(cchr* database)
{
   char         answer[maxline];
   char         tag[maxline];
   char         tfn[maxstring];
   int*         a;
   int*         go;
   int          cut;
   int          error;
   int          max_line_number;
   int          m;
   int          n;
   int          nn;
   int          nscores;
   int          o_sw;
   int          w_sw;
   int          option = 0;
   dbl*         weight;
   FILE*        ifp;
   FILE*        ofp;
   INFO_LINE*   hp;

   while (option == 0) {
      printf("%s\n%s\n%s\n%s\n%s\n%s\n\n",
         "Different kinds of totals can be computed:",
         "   0  instructions",
         "   1  ((cut k)) total",
         "   2  ((cut k)) weighted total",
         "   3  ((cut k)) weighted ordered total",
         "   4  quit");
      printf("Enter your option number: ");
      get_string_reply(answer);
      putchar('\n');
      if (*answer == 'i')
         option = 0;
      else if (*answer == 'q')
         option = 4;
      else
         option = extract_int(answer, 0, 4);
      if (option == 0)
         prn_c_t_s_info();
      if (option == 4)
         return 0;                                  /* no action taken */
   }
   tmpnam(tfn);
   startup(database, &ifp, tfn, &ofp);
   get_new_tag(tag, ifp);
   printf("\nHere is a snapshot of the input file.\n\n");
   snapshot(ifp);
   hp = cll_info_lines(ifp);
   n = max_line_number = cnt_ll_lines(hp);
   nn = n + 1;                                         /* size of go[] */
   do {
      do {
         printf("Here are the info lines extracted from the snapshot.\n\n");
         prn_ll_lines(hp);
         printf("%s\n%s",
            "Which score lines are needed for this computation?",
            "Input line numbers:  ");
         get_string_reply(answer);
         printf("\n");
         /***=
           *   Get the line numbers from answer.
           *   If there are no errors, then
           *      a[] contains the line numbers,
           *      m is the size of a[],
           *      a[] is in ascending order,
           *      and 1 <= a[0] && a[m-1] <= n is true.
         =***/
         get_line_numbers(&a, &m, &error, answer, max_line_number);
         if (error)
            printf("\nThere is an error in your input.  Do it again.\n\n");
      } while (error);
      assert(1 <= m && m <= n);
      go = (int *) gcalloc(nn, sizeof(int));
      get_go_array(a, m, go, nn, &error, hp, n);
      if (error)
         printf("Each go line has to be a score line.  Do it again.\n\n");
   } while (error);
   /***=
     *   Preliminaries are finished.
     *   Cuts and weights may be needed.
   =***/
   switch (option) {
   case 1:
      o_sw = 0;
      w_sw = 0;
      break;
   case 2:
      o_sw = 0;
      w_sw = 1;
      break;
   case 3:
      o_sw = 1;
      w_sw = 1;
      break;
   default:
      printf("PROGRAMMER ERROR: switch cannot get here.\n\n");
      exit(1);
   }
   weight = get_weight_array(&nscores, go, nn, hp, w_sw);
   rll_lines(hp);
   if (nscores > 1) {
      printf("\n%s%d\n%s",
         "Number of scores: ", nscores,
         "How many low scores do you wish to cut?  ");
      cut = get_int_reply(0, nscores - 1);
      printf("\n");
   }
   else
      cut = 0;
   printf("Computing totals ...\n\n");
   c_t_s_runrun(cut, go, nn, o_sw, tag, weight, nscores, ifp, ofp);
   wrapup(database, ifp, tfn, ofp);
   return 1;                                           /* action taken */
}
