#include "grade.h"

// ---
// A comment can occur at the end of a score line.
// See READ_ME for what constitutes a comment.
// [[It is difficult to describe a comment
// inside a comment.]]
//
// If no comment is present, s[0] = '\0'; otherwise nbsbc
// blank spaces are put into  s  followed by the comment.
// The int 1 gets returned if a comment was found;
// otherwise 0 get returned.
//
// ---
// Mnemonics:
//
//    nbsbc := number of blanks spaces before comments
//

int extract_comment(char* line, char* s)
{
   char*   p;
   int     go;

   *s = '\0';                              /* s starts as a blank line */
   for (p = strchr(line, ':'); *p != '\0'; ++p)
      go =    (*p == '/' && *(p + 1) == '/')
	   || (*p == '/' && *(p + 1) == '*')
	   || *p == '#';
      if (go) {
	 strcpy(s, blank_spaces(nbsbc));   /* space before the comment */
	 strcat(s, p);
	 remove_trailing_space(s);
	 return 1;
      }
   return 0;
}

// ---
// Saturday   13:18:25 PDT   18 October 1986
//
// Information has been requested from the user.
// The reply has been stored in answer as a stripped string.
// This function gets an integer from answer and stores it in n.
// The value of n must satisfy min <= n && n <= max .
//

int extract_int(char* answer, int min, int max)
{
   char   *p = answer;
   int    n;

   if (*answer == '\0') {
      printf("ERROR: You must input an integer:  ");
      return (get_int_reply(min, max));
   }
   if (*p == '+' || *p == '-')
      ++p;
   while (isdigit(*p))
      ++p;
   if (*p != '\0') {
      printf("ERROR: You must input an integer:  ");
      return (get_int_reply(min, max));
   }
   sscanf(answer, "%d", &n);
   if (n < min || n > max) {
      printf("ERROR: You must input an integer between %d and %d:  ", min, max);
      return (get_int_reply(min, max));
   }
   return (n);
}

// ---
// A score line must
//
//    begin with a one or more blanks or tabs
//    contain a : which ends the tag
//
// Comments, if any, must occur at the end of the score line.
//
// Scores are extracted from line and properly formatted in s.
//
// ---
// Mnemonics:
//
//    mnbsbs := minimum number of blank spaces before scores
//

void extract_scores(char* line, int max_fsfl, int max_rsfl, char* s)
{
   /*
   //  max_fsfl = max first score field length
   //  max_rsfl = max remaining scores field length
   */
   char    copy[maxline];
   char    t[maxline];
   char*   p;
   char*   q;
   int     go;
   int     i = 0;
   int     first = 1;
   int     j;
   int     max;
   int     n;                            /* i is reserved to index s[] */

   /***
     *   The variable max is only used to format the first score.
     *   Thereafter, max_rsfl is used.
   ***/
   max = (max_fsfl > max_rsfl) ? max_fsfl : max_rsfl;
   /*
   // It seems like an assertion should be used.
   */
   assert(strchr(line, ':') != NULL);
   strcpy(copy, strchr(line, ':'));
   for (p = copy + 1; *p != '\0'; ++p) {
      go =    (*p == '/' && *(p + 1) == '/')
	   || (*p == '/' && *(p + 1) == '*')
	   || *p == '#';
      if (go) {
	 *p = '\0';                /* discard comments in copy, if any */
	 break;
      }
   }
   p = copy;
   SET(p);
   while (*p != '\0') {
      for (j = 0, q = p; *q != '\0' && !isspace(*q); ++j, ++q)
	 t[j] = *q;
      t[j] = '\0';
      /* mnbsbs = minimum number of blank spaces before scores */
      if (first) {
	 first = 0;
	 n = max - strlen(t) + mnbsbs;
      }
      else
	 n = max_rsfl - strlen(t) + mnbsbs;
      for (j = 0; j < n; ++j)
	 s[i++] = ' ';
      for (q = t; *q != '\0'; ++q)
	 s[i++] = *q;
      SET(p);
   }
   s[i] = '\0';
}

// ---
// A score line is passed in.
// The tag is obtained from line by copying characters into s.
// Leading blanks and tabs are not copied.
// The first : character encountered ends the tag.
// Leading blanks are then inserted to properly format the tag in s.
//
//   max_tfl := max tag field length
//    mnbsbt := minimum number of blank spaces before the tag
//

void extract_tag(cchr* line, int max_tfl, char* s)
{
   char    t[maxline];
   cchr*   p;
   int     i = 0;
   int     j;
   int     n;

   for (p = line; *p == ' ' || *p == '\t'; ++p)
      ;
   for ( ; *p != ':'; ++p)
      s[i++] = *p;
   s[i++] = ':';
   s[i] = '\0';
   /*
   // Insert leading blanks into s.
   */
   n = mnbsbt + max_tfl - strlen(s);
   for (j = 0; j < n; ++j)
      t[j] = ' ';
   t[j] = '\0';                     /* t is a string of n blank spaces */
   strcat(t, s);
   strcpy(s, t);
   /*
   // s has no trailing blanks.
   */
   assert(!isspace(*(s + strlen(s) - 1)));
}
