#include "grade.h"

int is_nroff_he_line(char* line)
{
   char*   p;

   if (strncmp(line, ".he ", 4) != 0)
      return (0);
   for (p = line + 4; isspace(*p); ++p)
      ;
   if (*p != '\'')
      return (0);
   for (++p; *p != '\'' && *p != '\0'; ++p)
      ;
   if (*p != '\'')
      return (0);
   for (++p; *p != '\'' && *p != '\0'; ++p)
      ;
   if (*p != '\'')
      return (0);
   return (1);
}

// ---
// A blank line consists of white space.
// Return 1 if line is a blank line,
// and return 0 otherwise.
//

int isblank(char* line)
{
   char   *p = line;

   while (isspace(*p))
      ++p;
   if (*p == '\0')
      return 1;
   else
      return 0;
}

// ---
// 21 Nov 94
//
// The Registrar now uses a new name line.
// Here is an example:
//
//    Date: Mon, 21 Nov 1994 09:28:41 -0800
//    To: blufox@cats.ucsc.edu
//    From: script@cats.ucsc.edu (SCRIPT Center, x4682)
//
//    ^^^^,Do not remove or modify these lines. Enter evaluation text between them.
//    @@@@,MATH   012    01 ,55480,942,KELLEY,A.         
//    &&&&,AGCONGAY, ARLENE JO A, ,560590741,F,CW,SO,CMP1,    ,PL,55480,942
//    &&&&,ALLEN, JONATHAN EDWAR, ,569892903,M,OA,JR,CMP ,    ,PL,55480,942
//    &&&&,AMPON, RODERICK D    , ,559938474,M,MR,JR,CMP ,    ,PL,55480,942
//    &&&&,ARREDONDO, GUSTAVO   , ,617565846,M,CL,SO,PHY1,    ,LG,55480,942
//    &&&&,ARTETA, CARLOS OSCAR , ,621564796,M,MR,JR,ECON,    ,LG,55480,942
//    .....
//

// ---
// Saturday   23:26:15 PST   17 January 1987
//   
// The following must be recognized as name lines:
//   
//    NAME: BLUFOX,ABC ...
//    Name: BLUFOX, ABC ...
//    BLUFOX, ABC ...
//    BLU FOX, ABC ...
//

int isname(char* line)
{
   char*   p = line;
   int     go;

   go =    strncmp(line, "NAME:", 5) == 0
        || strncmp(line, "Name:", 5) == 0
        || strncmp(line, "&&&&,", 5) == 0;
   if (go)
      return 1;
   if (isupper(*p)) {
      for (++p; isupper(*p) || *p == ' '; ++p)
	 ;
      if (ispunct(*p))
	 return 1;
   }
   return 0;
}

// ---
// If the array of integers  w[0], ... , w[n-1]  is a permutation
// of the integers  1, ... , n  then the int value  1  is returned,
// otherwise the int value  0  is returned.
//

int ispermutation(int* w, int n)
{
   int   error = 0;
   int   i;
   int   j;

   for (i = 0; i < n && !error; ++i)
      if (w[i] < 1 || w[i] > n)
	 error = 1;
   for (i = 0; i < n - 1 && !error; ++i)
      for (j = i + 1; j < n && !error; ++j)
	 if (w[i] == w[j])
	    error = 1;
   return (!error);
}

// ---
// A score line begins with one or more blanks or tabs and
// contains a colon which ends the tag.
// Following the tag are one or more scores.
// Following the scores are zero or more comments.
//
// There are two kinds of score lines:
//
//    simple score lines
//    total score lines
//
// This function does not distinguish between the two kinds.
//

int isscore(cchr* line)
{
   if ((*line == ' ' || *line == '\t') && strchr(line, ':') != NULL)
      return 1;
   else
      return 0;
}

// ---
// A simple score line
//
//    begins with one or more blanks or tabs
//    contains a colon which ends the tag
//    and the tag does not contain any of the strings
//       "total"   "Total"   "TOTAL" .
//
// Following the tag are one or more scores.
// Following the scores are zero or more comments.
//

int issimple_score(cchr* line)
{
   char   copy[maxline];

   if ((*line == ' ' || *line == '\t') && strchr(line, ':') != NULL) {
      strcpy(copy, line);
      //
      // Discard the colon and the rest of the line.
      //
      *strchr(copy, ':') = '\0';
      if (vsindex(copy, "total") == NULL)
	 return 1;
   }
   return 0;
}

// ---
// A total score line
//
//    begins with one or more blanks or tabs
//    contains a colon which ends the tag
//    and the tag must contain one of the strings
//       "total"   "Total"   "TOTAL" .
//
// Checking for one of these strings is accomplised
// by passing line and "total" to vsindex().
//
// Following the tag is a score.
// Following the score are zero or more comments.
//

int istotal_score(cchr* line)
{
   char   copy[maxline];

   if ((*line == ' ' || *line == '\t') && strchr(line, ':') != NULL) {
      strcpy(copy, line);
      //
      // Discard the colon and the rest of the line.
      //
      *strchr(copy, ':') = '\0';
      if (vsindex(copy, "total") != NULL)
	 return 1;
   }
   return 0;
}
