// ---
// The following must be coordinated:
//
//    the cases in the switch statement in main()
//    the option values in get_option()
//
// The coordination used to be done via the options.h
// header file, but on 21 Jun 98 that header file was
// done in and the essential information included
// in grade.h.
//
// ---
// Quit := highest option number
//

#include "grade.h"

options   o;                                        // options are global

int main(int argc, char** argv)
{
   char   database[maxword];
   int    fwai;                                // first working arg index
   int    nwacl;                        // # of working args on cmnd line
   int    option;

   // ---
   // 23 Jun 97
   //
   // According to man -s 2 wait, this is
   // the function prototype for  wait():
   //
   //    #include <sys/types.h>
   //    #include <sys/wait.h>
   //
   //     pid_t wait(int *stat_loc);
   //
   
   int    status;

   int    action_cnt            = 0;
   int    error                 = 0;
   int*   e_ptr                 = &error;
   int    go_pretty_print       = 0;
   int    go_compute_statistics = 0;

   o.get_options(argc, argv);
   fwai = o.get_fwai();
   nwacl = o.get_nwacl();
   assert(fwai > 0 && fwai < argc);
   assert(nwacl == 0 || nwacl == 1);
   if (nwacl == 1) {
      strcpy(database, argv[fwai]);
      cout << "---\n"
	      "Checking the database ...\n\n";
      cout.flush();
      if (fork() == 0) {                              // fork() used here
         error = chk_for_errors(database);
         if (error)
            *e_ptr = error;                  // communicate to other fork
         exit(error);
      }
   }
   else
      *database = '\0';
   while ((option = get_option_from_list()) == 0)
      prn_main_info();
   if (*database == '\0' && option != Quit) {
      printf("Enter the name of your database:  ");
      get_string_reply(database);
      putchar('\n');
      error = chk_for_errors(database);
      if (error) {
	 printf("WHOOPS: Error reported - bye!\n\n");
         exit(1);
      }
   }
   if (option != Quit) {
      //
      // In a previous version of this program, I had written
      //
      //    union wait   status;
      //
      //    .....
      //
      //    wait(&status);      // correct usage?
      //
      // This stopped working at some point, probably because
      // the underlying operating system mechanisms changed.
      // To read about old Berkeley ways, give the
      // following command:
      //
      //    man -s 3b wait
      //
      // Here is a comment that I had written at
      // some point in the past:
      //
      //    // It seems that status.w_retcode does not
      //    // get reported correctly.  Why?  Since this
      //    // mechanism fails, the variable error is
      //    // used to communicate between the forks.
      //
      // The comment preceded the following code:
      //
      //    if (error || status.w_retcode != 0) {
      //       printf("ERROR reported - bye!\n\n");
      //       exit(1);
      //    }
      //
      wait(&status);
      if (error) {
         printf("ERROR reported - bye!\n\n");
         exit(1);
      }
      if (WIFEXITED(status) == 0) {
	 printf("%s%d%s",
	    "ERROR:\n"
	    "\n"
	    "   Child process may have terminated abnormally.\n"
	    "   Unfortunately, the man pages are confusing:\n"
	    "\n"
	    "      WIFEXITED(status) = ", WIFEXITED(status), "\n"
	    "\n");
         exit(1);
      }
   }
   for ( ; option != Quit; option = get_option_from_list()) {
      switch (option) {
      case Instructions:
         prn_main_info();
         break;
      case Enter_scores_or_lines:
         enter_scores_or_lines(database);
         ++action_cnt;
         go_pretty_print = 1;
         go_compute_statistics = 1;
         break;
      case Compute_total_scores:
         if (compute_total_scores(database) == 1) {
            ++action_cnt;
            go_pretty_print = 1;
            go_compute_statistics = 1;
         }
         break;
      case Assign_grades:
         assign_grades(database);
         ++action_cnt;
         break;
      case Compute_statistics:
         compute_statistics(database);
         ++action_cnt;
         go_compute_statistics = 0;
         break;
      case Pretty_print:
         pretty_print(database);
         ++action_cnt;
         go_pretty_print = 0;
         break;
      case Print_on_paper:
         print_on_paper(database);
         ++action_cnt;
         break;
      case Publish_for_students:
         publish(database);
         ++action_cnt;
         break;
      case Reorder_the_database:
         reorder(database);
         ++action_cnt;
         go_pretty_print = 1;
         break;
      case View:
         view(database);
         break;
      case Quit:                         // Quit =  highest option number
         break;                                  // do nothing, just quit
      default:
         printf("\n\nPROGRAMMER ERROR: Not supposed to get here!\n\n");
         exit(1);
      }
   }
   if (go_pretty_print)
      pretty_print(database);
   if (go_compute_statistics)
      compute_statistics(database);
   if (action_cnt > 0)
      printf("Grading complete: BYE!\n\n");
}
