#include "grade.h"

static no_yes   allowed_to_overwrite(cchr* ofn);
static void     chk_names(istream& is);
static void     get_answer(char* answer);

void publish(cchr* database)
{
   char       first_name[maxword];
   char       last_name[maxword];
   char       line[maxline];
   char       ofn[maxstring];                          // output filename
   char       next1;
   char       next2;
   char*      p;
   cchr*      q;
   int        n;
   int        n1;
   int        n2;
   ifstream   ifs;
   ofstream   ofs;

   assert(access(database, R_OK) == 0);
   cout << "---\n"
           "PUBLISH THE DATABASE FOR STUDENT PERUSAL:\n"
           "\n"
           "The database will be copied into another file\n"
           "with names reduced to two characters so that\n"
           "the database can be published online for\n"
           "student perusal.\n"
           "\n"
           "Input the name of the output file: ";
   get_answer(ofn);
   cout << "\n";
   if (access(ofn, F_OK) == 0 && allowed_to_overwrite(ofn) == no) {
      cout << "---\n"
	      "The database will not be published.\n"
	      "\n";
      cout.flush();
      return;                                                   // return
   }
   ifs.open(database);
   if (!ifs)
      error_exit_cannot_open(database);
   chk_names(ifs);
   grewind(ifs);
   ofs.open(ofn);
   if (!ofs)
      error_exit_cannot_open(ofn);
   //
   // ---
   // Publish two-letter names.
   //
   // ---
   // If a name is comprised of either one or two letters,
   // make the second letter X.  Thus, for example, the
   // name WU gets mildly disguised.
   //
   while (ifs.getline(line, maxline)) {
      if (!isname(line)) {
	 ofs << line << "\n";
	 continue;
      }
      //
      // Last name must be followed by a comma.
      // After the comma we expect a blank, but
      // we can manage without it.
      //
      strcpy(last_name, line);
      p = strchr(last_name, ',');
      assert(p != NULL);
      *p = '\0';
      //
      // It can happen that the name line looks like the following:
      //
      //    NAME: ACEDERA, ACHELLE LAMP, ,553872194,F,OA,JR,MCDB, ...
      //    NAME: ADAMS, TRACY RENEE   , ,552890640,F,CW,SR,BIOL, ...
      //
      // If this is the case, we want to get rid of "NAME: ".
      //
      p = strstr(last_name, "NAME:");
      if (p != NULL) {
	 for (p += 5; isspace(*p); ++p)
	    ;
	 assert(isupper(*p));
	 strcpy(last_name, p);
      }
      //
      // Now capture the first name.
      //
      q = strchr(line, ',');
      assert(q != NULL);
      for (++q; isspace(*q); ++q)
	 ;
      strcpy(first_name, q);
      //
      // There can be garbage at the end of the line as in
      // the ACEDERA example in the above comment, or perhaps
      // something like JOHN, JR.  To take care of such cases,
      // look for punctuation and kill the rest of the line.
      //
      for (p = first_name; *p != '\0' && !ispunct(*p); ++p)
	 ;
      if (ispunct(*p))
	 *p = '\0';
      //
      // Although it may not be necessary to do so,
      // we want to get rid of any ending blanks.
      // The ADAMS example in the above comment
      // shows that ending blanks can occur.
      //
      n = strlen(first_name);
      for (p = first_name + n - 1; p >= first_name && isspace(*p); --p)
	 ;
      assert(isupper(*p));
      *(p + 1) = '\0';
      // At this point we have the names, and
      // there should be no ending punctuation.
      //
      p = last_name;
      q = first_name;
      assert(isupper(*p));
      assert(isupper(*q));
      n1 = strlen(p);
      n2 = strlen(q);
      assert(isupper(*(p + n1 - 1)));
      assert(isupper(*(q + n2 - 1)));
      //
      // Select second characters.
      //
      next1 = (n1 > 2) ? *(p + 1) : 'X';
      next2 = (n2 > 2) ? *(q + 1) : 'X';
      //
      // Publish the abbreviated name.
      //
      ofs << *p << next1 << ", " << *q << next2 << "\n";
   }
   ifs.close();
   ofs.close();
}

static no_yes allowed_to_overwrite(cchr* ofn)
{
   char   answer[maxword];

   cout << "---\n"
	   "CAUTION: The file " << ofn << " exists.\n"
	   "\n"
	   "Can the file be overwritten? ";
   get_answer(answer);
   cout << endl;
   if (*answer == 'y' || *answer == 'Y')
      return yes;
   return no;
}

static void chk_names(istream& is)
{
   char    line[maxline];
   cchr*   p;

   assert(is);
   while (is.getline(line, maxline)) {
      if (!isname(line))
	 continue;
      p = strchr(line, ',');
      assert(p != NULL);
      if (*(p + 1) != ' ')
	 error_exit_unexpected_name_line(line);
   }
}

static void get_answer(char* answer)
{
   char   line[maxline];
   int    bad;

   do {
      cin.getline(line, maxline);
      assert(cin);
      istrstream   iss(line);
      iss >> answer;
      bad = !iss;
   } while (bad);
}
