/*
// remake() gets used in append_to_score_line().
//
// In append_to_score_line() name_line has just been written to stdout.
// Then the magic score line is found ((the one that needs to be remade))
// and the function remake(line, n, name_line, w) is executed.
//
// line := the magic score line that is to be remade
//
// Any comment at the end of line is stripped off and saved in s.
// Then line is stripped of any leading and trailing white space.
// At this point line is written to stdout with the cursor sitting
// at the end of the line.  Now the user enters n data items.
// The count for the correct number of data items is made and line
// is remade with the entered data appropriately appended.
// If a comment was saved, it must be reattached to line.
//
//    line      := the line that is to be remade
//    n         := the number of data items to be appended to line
//    w         := work space dynamically allocated in append_to_score_line()
//    name_line := name line
//
// If the user inputs the wrong number of data items,
// then a request to reenter the items is made,
// and name_line is reprinted.
//   
//    mnbsbt := minimum number of blank spaces before tags
//    mnbsbs := minimum number of blank spaces before scores
//    Mnbsbc := minimum number of blank spaces before comments
*/

#include "grade.h"

void remake(char* line, int n, char** w, char* name_line)
{
   char    input[maxline];
   char    gline[maxline];
   char    s[maxline];
   char*   p;
   int     ec;
   int     error;
   int     i;

   ec = extract_comment(line, s);                 /* save comment in s */
   if (ec)                                        /* extracted comment */
      strip_comment(line);
   strip(line);
   /***=
     *   At this point line has been stripped of its comment, if any,
     *   and of leading and trailing white space.  In general, line
     *   is a score line, so it is treated as such.  In particular,
     *   we remake the line with some leading blanks.
   =***/
   do {
      for (i = 0; i < mnbsbt; ++i)
	 fputc(' ', stdout);
      fputs(line, stdout);
      for (i = 0; i < mnbsbs; ++i)
	 fputc(' ', stdout);  /* cursor sitting at the end of the line */
      fgets(input, maxline, stdin);              /* get data from user */
      error = (word_cnt(input) != n);
      if (error) {
	 if (n == 1)
	    printf("\nOnly one score needed - do it again.\n\n");
	 else
	    printf("\n%s%d%s\n\n",
	       "Exactly ", n, " scores needed - do it again.");
	 printf("%s", name_line);
      }
   } while (error);                        /* user has to get it right */
   /***
     *   At this point the user has entered n data items
     *   which is now stored in the string input.
     *   This data now must be appended to line.
   ***/
   p = skip_space(input);
   for (i = 0; i < n; ++i) {
      sscanf(p,"%s", w[i]);
      SET(p);
   }
   /***
     *   Data items to be appended are now stored in
     *      w[0], ... , w[n-1]
     *   At this point we are ready to remake the line.
   ***/
   strcpy(gline, blank_spaces(mnbsbt));
   strcat(gline, line);
   for (i = 0; i < n; ++i) {               /* remake the line in gline */
      strcat(gline, blank_spaces(mnbsbs));
      strcat(gline, w[i]);
   }
   if (ec) {
      strcat(gline, blank_spaces(nbsbc));
      strcat(gline, s);                             /* comment is in s */
   }
   strcat(gline, "\n");
   strncpy(line, gline, maxline);
   if (strlen(gline) > maxline - 1) {
      printf("\n\n%s\n\n%s\n\n%s\n\n",
	 "SEVERE ERROR: The following line is too long:",
	 gline,
	 "The above line has been truncated in your database");
      line[maxline - 2] = '\n';
      line[maxline - 1] = '\0';
   }
}
