/*
// Reorder the database by name or by score.
//
// NAIL := name and info lines
//
// NAIL is a structure containing pointers to a name line
// and an array of info lines and one score.
// The following is excerpted from grade.h:
//      
//    typedef   struct s_nail   NAIL;
//
//    struct s_nail {
//       char*    name_line;
//       char**   info_line;
//       dbl      score;         /\* a score from one of the score lines *\/
//    };
//
// The header consists of all lines before the first name line.
// The body consists of all the remaining lines in the database,
// that is, all the name and info lines in the database.
//
// nil := number of info lines associated with a single name line
// nn  := number of name lines in the database
//     := number of students in the database
*/

#include "grade.h"

void reorder_by(char* name_or_score, cchr* database)
{
   char     line[maxline];
   char     tfn[maxstring];
   int      i;
   int      j;
   int      nail_cnt;
   int      nil;
   int      nn;
   int      body_cnt = 0;
   int      header_cnt = 0;
   int      line_cnt = 0;
   int      score_line_cnt = 0;
   FILE*    ifp;
   FILE*    ofp;
   LINE*    hp;
   LINE*    q;
   NAIL*    a;
   NAIL**   p;

   if (strcmp(name_or_score, "name") != 0
      && strcmp(name_or_score, "score") != 0) {
	 printf("ERROR: name_or_score must be \"name\" or \"score\"\n\n");
	 exit(1);
   }
   tmpnam(tfn);
   startup(database, &ifp, tfn, &ofp);
   hp = cll_nail(ifp);
   nail_cnt = cnt_ll_lines(hp);
   if (nail_cnt == 0) {
      printf("There are no name lines in your database.\n\n");
      gfclose(ifp);
      gfclose(ofp);
      return;
   }
   for (q = hp; q != NULL; q = q -> next)
      score_line_cnt += isscore(q -> line);
   if (strcmp(name_or_score, "score") == 0 && score_line_cnt == 0) {
      printf("There are no score lines in your database.\n\n");
      gfclose(ifp);
      gfclose(ofp);
      return;
   }
   nil = nail_cnt - 1;                         /* number of info lines */
   while (fgets(line, maxline, ifp) != NULL) {
      ++line_cnt;
      if (!isname(line))
	 ++header_cnt;
      else {
	 ++body_cnt;
	 while (fgets(line, maxline, ifp) != NULL) {
	    ++body_cnt;
	    ++line_cnt;
	 }
      }
   }
   rewind(ifp);
   nn = body_cnt / nail_cnt;       /* number of name lines in database */
   assert(header_cnt + body_cnt == line_cnt);
   assert(header_cnt + nn * nail_cnt == line_cnt);
   /*
   // Create working space.
   //
   // a[] := array of structures
   // p[] := array of pointers which point to the structures in a[]
   //
   // Reordering will be accomplished by sorting p[].
   */
   a = (NAIL *) gcalloc(nn, sizeof(NAIL));
   p = (NAIL **) gcalloc(nn, sizeof(NAIL *));
   for (i = 0; i < nn; ++i)
      p[i] = &a[i];
   /*
   // Write header lines to the outfile.
   */
   for (i = 0; i < header_cnt; ++i) {
      fgets(line, maxline, ifp);
      fputs(line, ofp);
   }
   /*
   // Create space in each a[i] and fill it.
   */
   for (i = 0; i < nn; ++i) {
      fgets(line, maxline, ifp);
      a[i].name_line = (char *) gcalloc(strlen(line) + 1, sizeof(char));
      strcpy(a[i].name_line, line);
      a[i].info_line = (char **) gcalloc(nil, sizeof(char *));
      for (j = 0; j < nil; ++j) {
	 fgets(line, maxline, ifp);
	 a[i].info_line[j] = (char *) gcalloc(strlen(line) + 1, sizeof(char));
	 strcpy(a[i].info_line[j], line);
      }
   }
   /*
   // Use qsort() to sort p[] by name or by score.
   // The comparison function is compare...().
   */
   if (strcmp(name_or_score, "name") == 0)
      qsort((char *) p, nn, sizeof(NAIL *), compare_lines);
   else {
      rewind(ifp);
      sort_by_score(a, p, nil, nn, ifp);
   }
   /*
   // Now that p[] is sorted, print to the outfile accordingly.
   */
   for (i = 0; i < nn; ++i) {
      fputs(p[i] -> name_line, ofp);
      for (j = 0; j < nil; ++j)
	 fputs(p[i] -> info_line[j], ofp);
   }
   /*
   // Free the work space.
   */
   rll_lines(hp);
   for (i = 0; i < nn; ++i) {
      free(a[i].name_line);
      for (j = 0; j < nil; ++j)
	 free(a[i].info_line[j]);
   }
   free(a);
   free(p);
   wrapup(database, ifp, tfn, ofp);
}
