// ---
// 21 Jun 98
//
// I need to break a long info line.
// Do I really want to do this here?
// Question:
//    If an info line gets broken into two info lines,
//    then the general shape of the database changes.
//    Will this be a problem?
//

// ---
// Tuesday   16:41:52 PDT   29 May 1990
//   
// This quarter Manyun is keeping track of the hmwk scores
// for M12: Linear Algebra in a separate database.  That means
// that the basic class database has to be reworked to incorporate
// the scores.  This function first checks to see that the basic
// database and the subsidiary database have identical name lines.
// Then the tags on score lines are checked.  The tag names
// in the database must be different from the tags in the
// subsidiary database.
//   
// Starting with this function, I am switching to ANSI C.
// Other functions get rewritten as I come to them.
//

#include "grade.h"

#define   ML     maxline
#define   QUIT   4

struct dbase_info {
   char**   dbase;                          // names of the two databases
   int*     nail_cnt;                           // name and info line cnt
   int      nn;                    // number of name lines, same for both
   FILE**   ifp;                         // pointers to the two databases
   LINE**   hp;                   // head pointers to name and info lines
};

typedef   struct dbase_info   dbase_info;

static void   break_info_line(cchr* database);
static int    chk_name_lines(dbase_info* dbi_ptr);
static void   chk_tags_in_score_lines(dbase_info* dbi_ptr);
static void   delete_lines(cchr* database);
static void   error_exit(char** dbase, char** s);
static void   merge_lines(cchr* database);
static void   merge_lines_kernel(dbase_info* dbi_ptr, FILE* ofp);

void rework(cchr* database)
{
   char   answer[maxline];
   int    option;

   cout << "Options to rework the database:\n"
	   "   1  break an info line\n"
	   "   2  delete info lines\n"
	   "   3  merge info lines\n"
	   "   4  quit\n"
	   "\n"
	   "Enter your option: ";
   cout.flush();
   get_string_reply(answer);
   cout << endl;
   if (*answer == 'b')
      option = 1;
   else if (*answer == 'd')
      option = 2;
   else if (*answer == 'm')
      option = 3;
   else if (*answer == 'q')
      option = QUIT;
   else
      option = extract_int(answer, 1, QUIT);
   switch (option) {
   case 1:
      break_info_line(database);
      break;
   case 2:
      delete_lines(database);
      break;
   case 3:
      merge_lines(database);
      break;
   case QUIT:
      break;
   default:
      printf("\nIn rework(): cannot get here - bye!\n\n");
      exit(1);
   }
}

static void break_info_line(cchr* database)
{
   char         line[maxline];
   char         new_tag[maxstring];
   char         old_tag[maxline];
   char         tfn[maxstring];
   char         w[maxword];
   cchr*        cp;
   char*        q;
   int          cnt;
   int          i;
   int          j;
   int          k;                                 // # of scores to keep
   int          m;                               // m is the magic number
   int          il_cnt;                                // info line count
   FILE*        ifp;
   FILE*        ofp;
   INFO_LINE*   hp;
   INFO_LINE*   p;
   ifstream     ifs;
   ofstream     ofs;

   tmpnam(tfn);
   startup(database, &ifp, tfn, &ofp);                  // bkup copy made
   hp = cll_info_lines(ifp);
   il_cnt = cnt_ll_lines(hp);
   if (il_cnt == 0) {
      cout << "---\n"
	      "WHOOPS: There are no info lines in your database!\n"
	      "\n";
      cout.flush();
      gfclose(ifp);
      gfclose(ofp);
      return;
   }
   cout << "Here is a snapshot of your database.\n\n";
   cout.flush();
   snapshot(ifp);
   cout << "Here are the info lines extracted from the snapshot.\n\n";
   cout.flush();
   cnt = 0;
   for (i = 1, p = hp; p != NULL; ++i, p = p -> next) {
      ++cnt;
      cout << setw(3) << i << p -> line;
   }
   cout << "\n"
           "Which line do you want to break? ";
   cout.flush();
   m = get_int_reply(1, cnt);                    // m is the magic number
   cout << endl;
   cnt = 0;
   for (i = 1, p = hp; p != NULL; ++i, p = p -> next) {
      ++cnt;
      if (cnt == m) {
	 if (!isscore(p -> line)) {
	    //
	    // Do it again.
	    //
	    gfclose(ifp);
	    gfclose(ofp);
	    gremove(tfn);
	    whoops_not_a_score_line(cnt, p -> line);
	    break_info_line(database);              // recursive fct call
	 }
	 break;
      }
   }
   //
   // Count the number of scores on the line.
   //
   strcpy(line, p -> line);
   cp = strchr(line, ':');
   assert(cp != NULL);
   istrstream   iss_scores(cp + 1);
   cnt = 0;
   while (iss_scores >> w)
      ++cnt;
   if (cnt < 2) {
      //
      // Do it again.
      //
      gfclose(ifp);
      gfclose(ofp);
      gremove(tfn);
      whoops_not_enough_scores(cnt, m, line);
      break_info_line(database);                    // recursive fct call
   }
   //
   // Capture the old tag.
   //
   strcpy(old_tag, line);
   q = strchr(old_tag, ':');
   assert(q != NULL);
   *(q + 1) = '\0';                        // delete the rest of the line
   //
   // Query the user.
   //
   cout << "Line " << m << " has " << cnt << " scores.\n"
	   "\n"
	   "How many scores do you wish to put on a new score line? ";
   cout.flush();
   k = get_int_reply(1, cnt - 1);
   cout << endl;
   k = cnt - k;                                // k = # of scores to keep
   assert(k > 0 && k < cnt);
   get_new_tag(new_tag, ifp);
   //
   // Do the rest via C++.
   // Reuse tfn.
   //
   gfclose(ifp);
   gfclose(ofp);
   ifs.open(database);
   ofs.open(tfn);
   assert(ifs);
   assert(ofs);
   while (ifs.getline(line, maxline)) {
      ofs << line << "\n";
      if (!isname(line))
	 continue;
      //
      // Write the info lines that preceed the magic line.
      //
      for (i = 1; i < m; ++i) {
	 ifs.getline(line, maxline);
	 ofs << line << "\n";
      }
      //
      // Capture the magic line and break it.
      //
      ifs.getline(line, maxline);
      cp = strchr(line, ':');
      assert(cp != NULL);
      istrstream   iss(cp + 1);
      ofs << old_tag;
      //
      // Write k scores on the current score line.
      //
      for (j = 1; j <= k; ++j) {
	 iss >> w;
	 ofs << " " << w;
      }
      ofs << "\n";
      //
      // Create a new score line and append the remaining scores.
      //
      ofs << new_tag;
      for (j = k + 1; j <= cnt; ++j) {
	 iss >> w;
	 ofs << " " << w;
      }
      ofs << "\n";
      //
      // Write the remaining info lines.
      //
      for (i = m + 1; i <= il_cnt; ++i) {
	 ifs.getline(line, maxline);
	 ofs << line << "\n";
      }
   }
   ifs.close();
   ofs.close();
   //
   // Overwrite the database with the tmp file.
   //
   ifs.open(tfn);
   ofs.open(database);
   while (ifs.getline(line, maxline))
      ofs << line << "\n";
   ifs.close();
   ofs.close();
}

static int chk_name_lines(dbase_info* dbi_ptr)
{
   char**   dbase;
   char     line[2][maxline];
   char*    s[2];
   char     wrk_line[maxline];
   int      cnt[2];
   int*     nail_cnt;
   int      nn;                                   // number of name lines
   long     offset;
   FILE**   ifp;

   dbase = dbi_ptr -> dbase;
   nail_cnt = dbi_ptr -> nail_cnt;
   ifp = dbi_ptr -> ifp;
   //
   // First run through the header lines.
   //
   while (offset = ftell(ifp[0]), (s[0] = fgets(wrk_line, ML, ifp[0])) != NULL)
      if (isname(wrk_line))
	 break;
   assert(s[0] != NULL);
   fseek(ifp[0], offset, 0);
   while (offset = ftell(ifp[1]), (s[1] = fgets(wrk_line, ML, ifp[1])) != NULL)
      if (isname(wrk_line))
	 break;
   assert(s[1] != NULL);
   fseek(ifp[1], offset, 0);
   //
   // Count all name and info lines
   // and check corresponding name lines.
   //
   cnt[0] = cnt[1] = 0;
   while ((s[0] = fgets(line[0], maxline, ifp[0])) != NULL) {
      ++cnt[0];
      if (isname(s[0])) {
	 strip(s[0]);
	 while ((s[1] = fgets(line[1], maxline, ifp[1])) != NULL) {
	    ++cnt[1];
	    if (isname(s[1])) {
	       strip(s[1]);
	       if (strcmp(s[0], s[1]) != 0)
		  error_exit(dbase, s);
	       break;
	    }
	 }
      }
   }
   assert(s[0] == NULL && s[1] != NULL);
   while ((s[1] = fgets(line[1], maxline, ifp[1])) != NULL) {
      ++cnt[1];
      assert(!isname(s[1]));
   }
   rewind(ifp[0]);
   rewind(ifp[1]);
   assert(cnt[0] % nail_cnt[0] == 0);
   assert(cnt[1] % nail_cnt[1] == 0);
   nn = cnt[0] / nail_cnt[0];
   assert(nn == cnt[1] / nail_cnt[1]);
   return nn;
}

static void delete_lines(cchr* database)
{
   char         answer[maxline];
   char         line[maxline];
   char         tfn[maxstring];
   char*        cp;
   int*         a;
   int          error;
   int*         go;
   int          i;
   int          j;
   int          n;
   int          tmp;
   int          il_cnt;                                // info line count
   long         offset;
   FILE*        ifp;
   FILE*        ofp;
   INFO_LINE*   hp;
   INFO_LINE*   p;

   tmpnam(tfn);
   startup(database, &ifp, tfn, &ofp);                  // bkup copy made
   hp = cll_info_lines(ifp);
   il_cnt = cnt_ll_lines(hp);
   if (il_cnt == 0) {
      printf("There are no info lines in your database!\n\n");
      gfclose(ifp);
      gfclose(ofp);
      return;
   }
   printf("Here is a snapshot of your database.\n\n");
   snapshot(ifp);
   printf("Here are the info lines extracted from the snapshot.\n\n");
   for (i = 1, p = hp; p != NULL; ++i, p = p -> next)
      printf("%3d%s", i, p -> line);
   putchar('\n');
   do {
      error = 0;
      printf("Which lines are to be deleted?  ");
      get_string_reply(answer);
      putchar('\n');
      for (cp = answer; isdigit(*cp) || isspace(*cp); ++cp)
	 ;
      if (*cp != '\0') {
	 error = 1;
	 printf("%s\n\n",
	    "ERROR: You must input positive integers only"
	    " - do it again.");
      }
   } while (error);
   n = word_cnt(answer);
   a = (int *) gcalloc(n, sizeof(int));
   --a;                                            // count from 1, not 0
   cp = answer;
   for (i = 1; i <= n; ++i) {
      while (isspace(*cp))
	 ++cp;
      assert(sscanf(cp, "%d", &a[i]) == 1);
      while (isdigit(*cp))
	 ++cp;
   }
   for (i = 1; i <= n; ++i)
      for (j = i + 1; j <= n; ++j)
	 if (a[i] > a[j]) {
	    tmp = a[i];
	    a[i] = a[j];
	    a[j] = tmp;
	 }
   error = 0;
   for (i = 1; i < n; ++i)
      if (a[i] == a[i + 1]) {
	 error = 1;
	 printf("ERROR: The numbers you enter must be unique.\n\n");
	 break;
      }
   if (a[1] < 1 || a[n] > il_cnt) {
      error = 1;
      printf("ERROR: You must enter numbers between 1 and %d.\n\n", il_cnt);
   }
   if (error) {
      gfclose(ifp);
      gfclose(ofp);
      free(a + 1);
      delete_lines(database);
      return;
   }
   go = (int *) gcalloc(il_cnt, sizeof(int));
   --go;                                           // count from 1, not 0
   i = 1;
   for (j = 1; j <= il_cnt; ++j)
      if (a[i] == j) {
	 ++i;
	 go[j] = 0;
      }
      else
	 go[j] = 1;
   assert(i == n + 1);
   /*
   // The work starts here.
   // We begin by running through the header lines.
   */
   while (offset = ftell(ifp), fgets(line, ML, ifp) != NULL && !isname(line))
      fputs(line, ofp);
   assert(isname(line));
   gfseek(ifp, offset, 0);
   while (fgets(line, maxline, ifp) != NULL) {
      fputs(line, ofp);                                      // name line
      for (i = 1; i <= il_cnt; ++i) {
	 fgets(line, maxline, ifp);
	 if (go[i])
	    fputs(line, ofp);
      }
   }
   free(a + 1);
   free(go + 1);
   wrapup(database, ifp, tfn, ofp);
}

static void error_exit(char** dbase, char** s)
{
   char   wrk[2][maxstring];
   int    max;
   int    n[2];

   assert(s[0] != NULL || s[1] != NULL);
   sprintf(wrk[0], "In %s: ", dbase[0]);
   sprintf(wrk[1], "In %s: ", dbase[1]);
   n[0] = strlen(wrk[0]);
   n[1] = strlen(wrk[1]);
   max = (n[0] > n[1]) ? n[0] : n[1];
   if (s[0] == NULL) {
      printf("%s\n\n%s%*s\n%s%*s%s\n\n",
	 "ERROR: Name lines do not agree:",
	 "   ", max, wrk[0],
	 "   ", max, wrk[1], s[1]);
   }
   else if (s[1] == NULL) {
      printf("%s\n\n%s%*s%s\n%s%*s\n\n",
	 "ERROR: Name lines do not agree:",
	 "   ", max, wrk[0], s[0],
	 "   ", max, wrk[1]);
   }
   else {
      printf("%s\n\n%s%*s%s\n%s%*s%s\n\n",
	 "ERROR: Name lines do not agree:",
	 "   ", max, wrk[0], s[0],
	 "   ", max, wrk[1], s[1]);
   }
   exit(1);
}

// ---
// Check that the tag names in the score lines in the
// subsidiary database do not clash with those
// in the basic database.
//

static void chk_tags_in_score_lines(dbase_info* dbi_ptr)
{
   char*         cp;
   int           i;
   FILE**        ifp;
   SCORE_LINE*   hp[2];
   SCORE_LINE*   p;
   SCORE_LINE*   q;

   ifp = dbi_ptr -> ifp;
   hp[0] = cll_score_lines(ifp[0]);
   for (p = hp[0]; p != NULL; p = p -> next) {
      if ((cp = strchr(p -> line, ':')) != NULL) {
	 *(cp + 1) = '\0';
	 strip(p -> line);
      }
   }
   hp[1] = cll_score_lines(ifp[1]);
   for (p = hp[1]; p != NULL; p = p -> next) {
      if ((cp = strchr(p -> line, ':')) != NULL) {
	 *(cp + 1) = '\0';
	 strip(p -> line);
      }
   }
   for (p = hp[0]; p != NULL; p = p -> next) {
      if (strchr(p -> line, ':') != NULL) {
	 for (q = hp[1]; q != NULL; q = q -> next) {
	    if (strchr(q -> line, ':') != NULL
	       &&  strcmp(p -> line, q -> line) == 0) {
		  printf("%s\n\n%s\n\n",
		     "ERROR: Tag names in the score lines clash.",
		     "Here are truncated score lines in the two databases:");
		  for (i = 1, p = hp[0]; p != NULL; ++i, p = p -> next)
		     printf("%5d  %s\n", i, p -> line);
		  putchar('\n');
		  for (i = 1, q = hp[1]; q != NULL; ++i, q = q -> next)
		     printf("%5d  %s\n", i, q -> line);
		  putchar('\n');
		  exit(1);
	    }
	 }
      }
   }
   rll_lines(hp[0]);
   rll_lines(hp[1]);
}

static void merge_lines(cchr* database)
{
   char         answer[maxstring];
   char         tfn[maxstring];
   char*        dbase[2];
   int          i;
   int          nail_cnt[2];
   FILE*        ifp[2];
   FILE*        ofp;
   LINE*        hp[2];
   dbase_info   dbi;

   dbase[0] = (char *) gcalloc(strlen(database) + 1, sizeof(char));
   strcpy(dbase[0], database);
   printf("%s\n\n%s",
      "Lines from a subsidiary database will be incorporated.",
      "Input the name of the subsidiary database:  ");
   get_string_reply(answer);
   putchar('\n');
   dbase[1] = (char *) gcalloc(strlen(answer) + 1, sizeof(char));
   strcpy(dbase[1], answer);
   if (chk_for_errors(dbase[1]) != 0)    // chk ending blank lines, other
      exit(1);
   tmpnam(tfn);
   startup(dbase[0], &ifp[0], tfn, &ofp);               // bkup copy made
   ifp[1] = gfopen(dbase[1], "r");
   for (i = 0; i < 2; ++i) {
      hp[i] = cll_nail(ifp[i]);
      nail_cnt[i] = cnt_ll_lines(hp[i]);
      if (nail_cnt[i] == 0) {
	 printf("There are no name lines in %s.\n\n", dbase[i]);
	 free(dbase[0]);
	 free(dbase[1]);
	 gfclose(ifp[0]);
	 gfclose(ifp[1]);
	 gfclose(ofp);
	 return;
      }
   }
   if (nail_cnt[1] == 1) {
      printf("There are no info lines in %s.\n\n", dbase[1]);
      free(dbase[0]);
      free(dbase[1]);
      gfclose(ifp[0]);
      gfclose(ifp[1]);
      gfclose(ofp);
      return;
   }
   dbi.dbase = dbase;
   dbi.nail_cnt = nail_cnt;
   dbi.ifp = ifp;
   dbi.hp = hp;
   dbi.nn = chk_name_lines(&dbi);            // nn = number of name lines
   chk_tags_in_score_lines(&dbi);
   merge_lines_kernel(&dbi, ofp);
   free(dbase[0]);
   free(dbase[1]);
   gfclose(ifp[0]);
   gfclose(ifp[1]);
   gfclose(ofp);
   copy(tfn, database);
   gremove(tfn);
}

static void merge_lines_kernel(dbase_info* dbi_ptr, FILE* ofp)
{
   char**   dbase;
   char     line[maxline];
   int      i;
   int      j;
   int*     nail_cnt;
   int      nn;
   long     offset;
   FILE**   ifp;
   LINE**   hp;

   dbase = dbi_ptr -> dbase;
   nail_cnt = dbi_ptr -> nail_cnt;
   hp = dbi_ptr -> hp;
   ifp = dbi_ptr -> ifp;
   nn = dbi_ptr -> nn;
   /*
   // First run through the header lines.
   */
   while (offset = ftell(ifp[0]), fgets(line, maxline, ifp[0]) != NULL) {
      if (!isname(line))
	 fputs(line, ofp);               // write header lines to outfile
      else
	 break;
   }
   fseek(ifp[0], offset, 0);
   while (offset = ftell(ifp[1]), fgets(line, maxline, ifp[1]) != NULL) {
      if (isname(line))                           // discard header lines
	 break;
   }
   fseek(ifp[1], offset, 0);
   for (i = 0; i < nn; ++i) {
      for (j = 0; j < nail_cnt[0]; ++j) {
	 fgets(line, maxline, ifp[0]);
	 fputs(line, ofp);
      }
      for (j = 0; j < nail_cnt[1]; ++j) {
	 fgets(line, maxline, ifp[1]);
	 if (j > 0)                              // discard the name line
	    fputs(line, ofp);
      }
   }
}
